package com.co.finder.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import javafx.fxml.FXML;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.stage.FileChooser;

public class ControllerFind {

	@FXML
	private TextArea resultados;
	@FXML
	private CheckBox esMayuscula;
	@FXML
	private TextField expresionRegular;
	@FXML
	private TextArea busquedaTexto;

	// Busca el texto
	@FXML
	private void SearchText() {

		resultados.setText("");
		final String[] lines = busquedaTexto.getText().split("\n");
		final int lengthLines = lines.length;

		final String regularExpresion = esMayuscula.isSelected() ? expresionRegular.getText().toUpperCase()
				: expresionRegular.getText();
		final Pattern pattern = Pattern.compile("^" + regularExpresion + "$");

		List<DtoLines> results = IntStream.rangeClosed(1, lengthLines).boxed().map(indice -> {
			return new DtoLines(indice, lines[indice - 1]);
		}).filter(line -> {
			return !line.getTexto().isEmpty();
		}).filter(line -> pattern.matcher(line.getTexto()).find()).collect(Collectors.toList());

		if (results.size() > 0) {
			results.forEach(resultado -> {
				resultados.appendText("La cadena " + resultado.getTexto() + " se encuentra en la linea "
						+ resultado.getLinea() + "\n");
			});
		} else {
			resultados.setText("No se encontr la cadena");
		}
	}

	// Carga archivo de texto
	@FXML
	private void loadText() throws FileNotFoundException, IOException {
		FileChooser fileChooser = new FileChooser();
		fileChooser.setTitle("Buscar texto");
		fileChooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("Text Files", "*.txt"));

		File fileReader = fileChooser.showOpenDialog(busquedaTexto.getScene().getWindow());

		try (InputStream fileIn = new FileInputStream(fileReader)) {
			String texto = new BufferedReader(new InputStreamReader(fileIn, Charset.forName("UTF-8"))).lines()
					.parallel().collect(Collectors.joining("\n"));
			busquedaTexto.setText(texto);
		}
	}

}

class DtoLines {

	private final int linea;
	private final String texto;

	public DtoLines(int linea, String texto) {
		this.linea = linea;
		this.texto = texto;
	}

	public int getLinea() {
		return linea;
	}

	public String getTexto() {
		return texto;
	}

}
